@@ -15,13 +15,43 @@
 import netaddr
 
 from neutron_lib import constants as const
+from neutron_lib import exceptions as n_exceptions
 from oslo_config import cfg
 
-
-from networking_ovn.common import config
+from networking_ovn._i18n import _
 from networking_ovn.common import constants as ovn_const
 from networking_ovn.common import utils
 
+# Convert the protocol number from integer to strings because that's
+# how Neutron will pass it to us
+PROTOCOL_NAME_TO_NUM_MAP = {k: str(v) for k, v in
+                            const.IP_PROTOCOL_MAP.items()}
+# Create a map from protocol numbers to names
+PROTOCOL_NUM_TO_NAME_MAP = {v: k for k, v in
+                            PROTOCOL_NAME_TO_NUM_MAP.items()}
+
+# Group of transport protocols supported
+TRANSPORT_PROTOCOLS = (const.PROTO_NAME_TCP,
+                       const.PROTO_NAME_UDP,
+                       const.PROTO_NAME_SCTP,
+                       PROTOCOL_NAME_TO_NUM_MAP[const.PROTO_NAME_TCP],
+                       PROTOCOL_NAME_TO_NUM_MAP[const.PROTO_NAME_UDP],
+                       PROTOCOL_NAME_TO_NUM_MAP[const.PROTO_NAME_SCTP])
+
+# Group of versions of the ICMP protocol supported
+ICMP_PROTOCOLS = (const.PROTO_NAME_ICMP,
+                  const.PROTO_NAME_IPV6_ICMP,
+                  const.PROTO_NAME_IPV6_ICMP_LEGACY,
+                  PROTOCOL_NAME_TO_NUM_MAP[const.PROTO_NAME_ICMP],
+                  PROTOCOL_NAME_TO_NUM_MAP[const.PROTO_NAME_IPV6_ICMP],
+                  PROTOCOL_NAME_TO_NUM_MAP[const.PROTO_NAME_IPV6_ICMP_LEGACY])
+
+
+class ProtocolNotSupported(n_exceptions.NeutronException):
+    message = _('The protocol "%(protocol)s" is not supported. Valid '
+                'protocols are: %(valid_protocols); or protocol '
+                'numbers ranging from 0 to 255.')
+
 
 def is_sg_enabled():
     return cfg.CONF.SECURITYGROUP.enable_security_group
@@ -58,47 +88,50 @@ def acl_remote_ip_prefix(r, ip_version):
                                 r['remote_ip_prefix'])
 
 
+def _get_protocol_number(protocol):
+    if protocol is None:
+        return
+    try:
+        protocol = int(protocol)
+        if protocol >= 0 and protocol <= 255:
+            return str(protocol)
+    except (ValueError, TypeError):
+        protocol = PROTOCOL_NAME_TO_NUM_MAP.get(protocol)
+        if protocol is not None:
+            return protocol
+
+    raise ProtocolNotSupported(
+        protocol=protocol, valid_protocols=', '.join(PROTOCOL_NAME_TO_NUM_MAP))
+
+
 def acl_protocol_and_ports(r, icmp):
-    protocol = None
     match = ''
-    if r['protocol'] in ('tcp', 'udp',
-                         str(const.PROTO_NUM_TCP),
-                         str(const.PROTO_NUM_UDP)):
-        # OVN expects the protocol name not number
-        if r['protocol'] == str(const.PROTO_NUM_TCP):
-            protocol = 'tcp'
-        elif r['protocol'] == str(const.PROTO_NUM_UDP):
-            protocol = 'udp'
-        else:
-            protocol = r['protocol']
-        port_match = '%s.dst' % protocol
-    elif r.get('protocol') in (const.PROTO_NAME_ICMP,
-                               const.PROTO_NAME_IPV6_ICMP,
-                               const.PROTO_NAME_IPV6_ICMP_LEGACY,
-                               str(const.PROTO_NUM_ICMP),
-                               str(const.PROTO_NUM_IPV6_ICMP)):
-        protocol = icmp
-        port_match = '%s.type' % icmp
-    if protocol:
+    protocol = _get_protocol_number(r.get('protocol'))
+    if protocol is None:
+        return match
+
+    min_port = r.get('port_range_min')
+    max_port = r.get('port_range_max')
+    if protocol in TRANSPORT_PROTOCOLS:
+        protocol = PROTOCOL_NUM_TO_NAME_MAP[protocol]
         match += ' && %s' % protocol
-        # If min or max are set to -1, then we just treat it like it wasn't
-        # specified at all and don't match on it.
-        min_port = -1 if r['port_range_min'] is None else r['port_range_min']
-        max_port = -1 if r['port_range_max'] is None else r['port_range_max']
-        if protocol != icmp:
-            if (min_port > -1 and min_port == max_port):
-                match += ' && %s == %d' % (port_match, min_port)
-            else:
-                if min_port > -1:
-                    match += ' && %s >= %d' % (port_match, min_port)
-                if max_port > -1:
-                    match += ' && %s <= %d' % (port_match, max_port)
-        # It's invalid to create security group rule for ICMP and ICMPv6 with
-        # ICMP(v6) code but without ICMP(v6) type.
-        elif protocol == icmp and min_port > -1:
-            match += ' && %s == %d' % (port_match, min_port)
-            if max_port > -1:
-                match += ' && %s.code == %s' % (icmp, max_port)
+        if min_port is not None and min_port == max_port:
+            match += ' && %s.dst == %d' % (protocol, min_port)
+        else:
+            if min_port is not None:
+                match += ' && %s >= %d' % (protocol, min_port)
+            if max_port is not None:
+                match += ' && %s <= %d' % (protocol, max_port)
+    elif protocol in ICMP_PROTOCOLS:
+        protocol = icmp
+        match += ' && %s' % protocol
+        if min_port is not None:
+            match += ' && %s.type == %d' % (protocol, min_port)
+        if max_port is not None:
+            match += ' && %s.code == %d' % (protocol, max_port)
+    else:
+        match += ' && ip.proto == %s' % protocol
+
     return match
 
 
@@ -135,23 +168,24 @@ def add_sg_rule_acl_for_port(port, r, match):
     return acl
 
 
-def add_acl_dhcp(port, subnet):
-    # Allow DHCP responses through from source IPs on the local subnet.
-    # We do this even if DHCP isn't enabled for the subnet.  It could be
-    # enabled later. We could hook into handling when it's enabled/disabled
-    # for a subnet, but this only used when OVN native DHCP is disabled.
+def add_acl_dhcp(port, subnet, ovn_dhcp=True):
+    # Allow DHCP requests for OVN native DHCP service, while responses are
+    # allowed in ovn-northd.
+    # Allow both DHCP requests and responses to pass for other DHCP services.
+    # We do this even if DHCP isn't enabled for the subnet
     acl_list = []
-    acl = {"lswitch": utils.ovn_name(port['network_id']),
-           "lport": port['id'],
-           "priority": ovn_const.ACL_PRIORITY_ALLOW,
-           "action": ovn_const.ACL_ACTION_ALLOW,
-           "log": False,
-           "direction": 'to-lport',
-           "match": ('outport == "%s" && ip4 && ip4.src == %s && '
-                     'udp && udp.src == 67 && udp.dst == 68'
-                     ) % (port['id'], subnet['cidr']),
-           "external_ids": {'neutron:lport': port['id']}}
-    acl_list.append(acl)
+    if not ovn_dhcp:
+        acl = {"lswitch": utils.ovn_name(port['network_id']),
+               "lport": port['id'],
+               "priority": ovn_const.ACL_PRIORITY_ALLOW,
+               "action": ovn_const.ACL_ACTION_ALLOW,
+               "log": False,
+               "direction": 'to-lport',
+               "match": ('outport == "%s" && ip4 && ip4.src == %s && '
+                         'udp && udp.src == 67 && udp.dst == 68'
+                         ) % (port['id'], subnet['cidr']),
+               "external_ids": {'neutron:lport': port['id']}}
+        acl_list.append(acl)
     acl = {"lswitch": utils.ovn_name(port['network_id']),
            "lport": port['id'],
            "priority": ovn_const.ACL_PRIORITY_ALLOW,
@@ -159,8 +193,7 @@ def add_acl_dhcp(port, subnet):
            "log": False,
            "direction": 'from-lport',
            "match": ('inport == "%s" && ip4 && '
-                     '(ip4.dst == 255.255.255.255 || '
-                     'ip4.dst == %s) && '
+                     'ip4.dst == {255.255.255.255, %s} && '
                      'udp && udp.src == 68 && udp.dst == 67'
                      ) % (port['id'], subnet['cidr']),
            "external_ids": {'neutron:lport': port['id']}}
@@ -297,20 +330,19 @@ def add_acls(plugin, admin_context, port, sg_cache, subnet_cache):
     # Drop all IP traffic to and from the logical port by default.
     acl_list += drop_all_ip_traffic_for_port(port)
 
-    # Add DHCP ACLs if not using OVN native DHCP.
-    if not config.is_ovn_dhcp():
-        port_subnet_ids = set()
-        for ip in port['fixed_ips']:
-            if netaddr.IPNetwork(ip['ip_address']).version != 4:
-                continue
-            subnet = _get_subnet_from_cache(plugin,
-                                            admin_context,
-                                            subnet_cache,
-                                            ip['subnet_id'])
-            # Ignore duplicate DHCP ACLs for the subnet.
-            if subnet['id'] not in port_subnet_ids:
-                acl_list += add_acl_dhcp(port, subnet)
-                port_subnet_ids.add(subnet['id'])
+    # Add DHCP ACLs.
+    port_subnet_ids = set()
+    for ip in port['fixed_ips']:
+        if netaddr.IPNetwork(ip['ip_address']).version != 4:
+            continue
+        subnet = _get_subnet_from_cache(plugin,
+                                        admin_context,
+                                        subnet_cache,
+                                        ip['subnet_id'])
+        # Ignore duplicate DHCP ACLs for the subnet.
+        if subnet['id'] not in port_subnet_ids:
+            acl_list += add_acl_dhcp(port, subnet, True)
+            port_subnet_ids.add(subnet['id'])
 
     # We create an ACL entry for each rule on each security group applied
     # to this port.
