@@ -12,33 +12,30 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import fixtures
-import mock
 import time
 
-from neutron_lib import constants
+import fixtures
+import mock
+from neutron.plugins.ml2 import config
+from neutron.plugins.ml2.drivers import type_geneve  # noqa
+from neutron.tests.unit.plugins.ml2 import test_plugin
+from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import uuidutils
+from ovsdbapp.backend.ovs_idl import command
+from ovsdbapp.backend.ovs_idl import connection
+from ovsdbapp.backend.ovs_idl import transaction
 
-from neutron.agent.ovsdb import impl_idl
-from neutron.agent.ovsdb.native import commands
-from neutron.plugins.ml2 import config
-from neutron.plugins.ml2.drivers import type_geneve  # noqa
-from neutron.tests.unit.plugins.ml2 import test_plugin
-
-from networking_ovn._i18n import _LE
 from networking_ovn.ovsdb import impl_idl_ovn
 from networking_ovn.ovsdb import ovsdb_monitor
 from networking_ovn.tests.functional.resources import process
 
-PLUGIN_NAME = ('networking_ovn.plugin.OVNPlugin')
-
 LOG = log.getLogger(__name__)
 
 
-class AddFakeChassisCommand(commands.BaseCommand):
+class AddFakeChassisCommand(command.BaseCommand):
     """Add a fake chassis in OVN SB DB for functional test."""
 
     def __init__(self, api, name, ip, **columns):
@@ -89,6 +86,17 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         self._start_ovsdb_server_and_idls()
 
     def tearDown(self):
+        # Set Mock() to idl to avoid SSL file access errors.
+        # This is because, destroying temporary directory containing SSL files
+        # is earlier than terminating thread in Connection() object that
+        # the errors are likely to occur in the short period of time.
+        # However, Connection() does not provider a stop method for run(), we
+        # replace idl with Mock() to avoid accessing.
+        if self._ovsdb_protocol == 'ssl':
+            impl_idl_ovn.OvsdbNbOvnIdl.ovsdb_connection.idl = mock.Mock()
+            impl_idl_ovn.OvsdbSbOvnIdl.ovsdb_connection.idl = mock.Mock()
+            self.monitor_nb_idl_con.idl = mock.Mock()
+            self.monitor_sb_idl_con.idl = mock.Mock()
         # Need to set OvsdbNbOvnIdl.ovsdb_connection and
         # OvsdbSbOvnIdl.ovsdb_connection to None.
         # This is because, when the test worker runs the next functional test
@@ -98,6 +106,13 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         impl_idl_ovn.OvsdbSbOvnIdl.ovsdb_connection = None
         super(TestOVNFunctionalBase, self).tearDown()
 
+    @property
+    def _ovsdb_protocol(self):
+        return self.get_ovsdb_server_protocol()
+
+    def get_ovsdb_server_protocol(self):
+        return 'unix'
+
     def _start_ovsdb_server_and_idls(self):
         self.temp_dir = self.useFixture(fixtures.TempDir()).path
         # Start 2 ovsdb-servers one each for OVN NB DB and OVN SB DB
@@ -105,15 +120,21 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         # events.
         self.ovsdb_server_mgr = self.useFixture(
             process.OvsdbServer(self.temp_dir, self.OVS_INSTALL_SHARE_PATH,
-                                ovn_nb_db=True, ovn_sb_db=True))
-        cfg.CONF.set_override(
-            'ovn_nb_connection',
-            self.ovsdb_server_mgr.get_ovsdb_connection_path(),
-            'ovn')
-        cfg.CONF.set_override(
-            'ovn_sb_connection',
-            self.ovsdb_server_mgr.get_ovsdb_connection_path(db_type='sb'),
-            'ovn')
+                                ovn_nb_db=True, ovn_sb_db=True,
+                                protocol=self._ovsdb_protocol))
+        set_cfg = cfg.CONF.set_override
+        set_cfg('ovn_nb_connection',
+                self.ovsdb_server_mgr.get_ovsdb_connection_path(), 'ovn')
+        set_cfg('ovn_sb_connection',
+                self.ovsdb_server_mgr.get_ovsdb_connection_path(
+                    db_type='sb'), 'ovn')
+        set_cfg('ovn_nb_private_key', self.ovsdb_server_mgr.private_key, 'ovn')
+        set_cfg('ovn_nb_certificate', self.ovsdb_server_mgr.certificate, 'ovn')
+        set_cfg('ovn_nb_ca_cert', self.ovsdb_server_mgr.ca_cert, 'ovn')
+        set_cfg('ovn_sb_private_key', self.ovsdb_server_mgr.private_key, 'ovn')
+        set_cfg('ovn_sb_certificate', self.ovsdb_server_mgr.certificate, 'ovn')
+        set_cfg('ovn_sb_ca_cert', self.ovsdb_server_mgr.ca_cert, 'ovn')
+
         num_attempts = 0
         # 5 seconds should be more than enough for the transaction to complete
         # for the test cases.
@@ -130,14 +151,16 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         #     ML2 OVN driver scope to test scenarios like ovn_nb_sync.
         while num_attempts < 3:
             try:
-                self.monitor_nb_idl_con = ovsdb_monitor.OvnBaseConnection(
+                _idlnb = ovsdb_monitor.BaseOvnIdl.from_server(
                     self.ovsdb_server_mgr.get_ovsdb_connection_path(),
-                    60, 'OVN_Northbound')
+                    'OVN_Northbound')
+                self.monitor_nb_idl_con = connection.Connection(
+                    idl=_idlnb, timeout=60)
                 self.monitor_nb_idl_con.start()
                 self.monitor_nb_db_idl = self.monitor_nb_idl_con.idl
                 break
             except Exception:
-                LOG.exception(_LE("Error connecting to the OVN_Northbound DB"))
+                LOG.exception("Error connecting to the OVN_Northbound DB")
                 num_attempts += 1
                 time.sleep(1)
 
@@ -149,15 +172,16 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         #  - Update chassis columns etc.
         while num_attempts < 3:
             try:
-                self.monitor_sb_idl_con = ovsdb_monitor.OvnBaseConnection(
-                    self.ovsdb_server_mgr.get_ovsdb_connection_path(
-                        db_type='sb'),
-                    60, 'OVN_Southbound')
+                _idlsb = ovsdb_monitor.BaseOvnIdl.from_server(
+                    self.ovsdb_server_mgr.get_ovsdb_connection_path('sb'),
+                    'OVN_Southbound')
+                self.monitor_sb_idl_con = connection.Connection(
+                    idl=_idlsb, timeout=60)
                 self.monitor_sb_idl_con.start()
                 self.monitor_sb_db_idl = self.monitor_sb_idl_con.idl
                 break
             except Exception:
-                LOG.exception(_LE("Error connecting to the OVN_Southbound DB"))
+                LOG.exception("Error connecting to the OVN_Southbound DB")
                 num_attempts += 1
                 time.sleep(1)
 
@@ -172,18 +196,23 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
 
     def nb_idl_transaction(self, fake_api, check_error=False, log_errors=True,
                            **kwargs):
-        return impl_idl.Transaction(fake_api, self.monitor_nb_idl_con, 60,
-                                    check_error, log_errors)
+        return transaction.Transaction(fake_api, self.monitor_nb_idl_con, 60,
+                                       check_error, log_errors)
 
     def sb_idl_transaction(self, fake_api, check_error=False, log_errors=True,
                            **kwargs):
-        return impl_idl.Transaction(fake_api, self.monitor_sb_idl_con, 60,
-                                    check_error, log_errors)
+        return transaction.Transaction(fake_api, self.monitor_sb_idl_con, 60,
+                                       check_error, log_errors)
 
     def restart(self):
         if self.ovsdb_server_mgr:
             self.ovsdb_server_mgr.stop()
 
+        if self._ovsdb_protocol == 'ssl':
+            impl_idl_ovn.OvsdbNbOvnIdl.ovsdb_connection.idl = mock.Mock()
+            impl_idl_ovn.OvsdbSbOvnIdl.ovsdb_connection.idl = mock.Mock()
+            self.monitor_nb_idl_con.idl = mock.Mock()
+            self.monitor_sb_idl_con.idl = mock.Mock()
         impl_idl_ovn.OvsdbNbOvnIdl.ovsdb_connection = None
         impl_idl_ovn.OvsdbSbOvnIdl.ovsdb_connection = None
         self.mech_driver._nb_ovn = None
