@@ -14,21 +14,24 @@
 
 import mock
 
+from neutron.services.segments import db as segments_db
+from neutron.tests.unit.api import test_extensions
+from neutron.tests.unit.extensions import test_extraroute
+from neutron.tests.unit.extensions import test_securitygroup
+from neutron_lib.api.definitions import l3
+from neutron_lib import constants
+from neutron_lib import context
+from neutron_lib.plugins import directory
 from oslo_utils import uuidutils
+from ovsdbapp.backend.ovs_idl import idlutils
 
 from networking_ovn.common import acl as acl_utils
+from networking_ovn.common import config as ovn_config
 from networking_ovn.common import constants as ovn_const
 from networking_ovn.common import utils
 from networking_ovn import ovn_db_sync
 from networking_ovn.ovsdb import commands as cmd
 from networking_ovn.tests.functional import base
-from neutron.agent.ovsdb.native import idlutils
-from neutron import context
-from neutron.services.segments import db as segments_db
-from neutron.tests.unit.api import test_extensions
-from neutron.tests.unit.extensions import test_extraroute
-from neutron_lib import constants
-from neutron_lib.plugins import directory
 
 
 class TestOvnNbSync(base.TestOVNFunctionalBase):
@@ -37,11 +40,14 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         super(TestOvnNbSync, self).setUp()
         ext_mgr = test_extraroute.ExtraRouteTestExtensionManager()
         self.ext_api = test_extensions.setup_extensions_middleware(ext_mgr)
+        sg_mgr = test_securitygroup.SecurityGroupTestExtensionManager()
+        self._sg_api = test_extensions.setup_extensions_middleware(sg_mgr)
         self.create_lswitches = []
         self.create_lswitch_ports = []
         self.create_lrouters = []
         self.create_lrouter_ports = []
         self.create_lrouter_routes = []
+        self.create_lrouter_nats = []
         self.update_lrouter_ports = []
         self.create_acls = []
         self.delete_lswitches = []
@@ -49,6 +55,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         self.delete_lrouters = []
         self.delete_lrouter_ports = []
         self.delete_lrouter_routes = []
+        self.delete_lrouter_nats = []
         self.delete_acls = []
         self.create_address_sets = []
         self.delete_address_sets = []
@@ -63,8 +70,18 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         self.lport_dhcpv6_disabled = {}
         self.missed_dhcp_options = []
         self.dirty_dhcp_options = []
+        self.lport_dhcp_ignored = []
+        self.match_old_mac_dhcp_subnets = []
+        ovn_config.cfg.CONF.set_override('ovn_metadata_enabled', False,
+                                         group='ovn')
 
-    def _create_resources(self):
+    def _api_for_resource(self, resource):
+        if resource in ['security-groups']:
+            return self._sg_api
+        else:
+            return super(TestOvnNbSync, self)._api_for_resource(resource)
+
+    def _create_resources(self, restart_ovsdb_processes=False):
         n1 = self._make_network(self.fmt, 'n1', True)
         res = self._create_subnet(self.fmt, n1['network']['id'],
                                   '10.0.0.0/24')
@@ -96,10 +113,12 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                 n1_s1['subnet']['id'])['uuid'])
         update_port_ids_v4 = []
         update_port_ids_v6 = []
+        n1_port_dict = {}
         for p in ['p1', 'p2', 'p3', 'p4', 'p5', 'p6', 'p7']:
             port = self._make_port(self.fmt, n1['network']['id'],
                                    name='n1-' + p,
                                    device_owner='compute:None')
+            n1_port_dict[p] = port['port']['id']
             lport_name = port['port']['id']
             lswitch_name = 'neutron-' + n1['network']['id']
             if p == 'p1':
@@ -255,6 +274,24 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
             port_req = self.new_update_request('ports', data, port_id)
             port_req.get_response(self.api)
 
+        # External network and subnet
+        e1 = self._make_network(self.fmt, 'e1', True,
+                                arg_list=('router:external',
+                                          'provider:network_type',
+                                          'provider:physical_network'),
+                                **{'router:external': True,
+                                   'provider:network_type': 'flat',
+                                   'provider:physical_network': 'public'})
+        self.assertEqual(True, e1['network']['router:external'])
+        self.assertEqual('flat', e1['network']['provider:network_type'])
+        self.assertEqual('public', e1['network']['provider:physical_network'])
+        res = self._create_subnet(self.fmt, e1['network']['id'],
+                                  '100.0.0.0/24', gateway_ip='100.0.0.254',
+                                  allocation_pools=[{'start': '100.0.0.2',
+                                                     'end': '100.0.0.253'}],
+                                  enable_dhcp=False)
+        e1_s1 = self.deserialize(self.fmt, res)
+
         self.create_lswitches.append('neutron-' + uuidutils.generate_uuid())
         self.create_lswitch_ports.append(('neutron-' +
                                           uuidutils.generate_uuid(),
@@ -263,11 +300,21 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                                           uuidutils.generate_uuid(),
                                           'neutron-' + n1['network']['id']))
         self.delete_lswitches.append('neutron-' + n2['network']['id'])
+        self.delete_lswitch_ports.append(
+            (utils.ovn_provnet_port_name(e1['network']['id']),
+             utils.ovn_name(e1['network']['id'])))
 
         r1 = self.l3_plugin.create_router(
             self.context,
-            {'router': {'name': 'r1', 'admin_state_up': True,
-                        'tenant_id': self._tenant_id}})
+            {'router': {
+                'name': 'r1', 'admin_state_up': True,
+                'tenant_id': self._tenant_id,
+                'external_gateway_info': {
+                    'enable_snat': True,
+                    'network_id': e1['network']['id'],
+                    'external_fixed_ips': [
+                        {'ip_address': '100.0.0.2',
+                         'subnet_id': e1_s1['subnet']['id']}]}}})
         self.l3_plugin.add_router_interface(
             self.context, r1['id'], {'subnet_id': n1_s1['subnet']['id']})
         r1_p2 = self.l3_plugin.add_router_interface(
@@ -281,12 +328,36 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                                           n1_s2['subnet']['gateway_ip']))
         self.delete_lrouter_ports.append(('lrp-' + r1_p3['port_id'],
                                           'neutron-' + r1['id']))
+        self.delete_lrouter_ports.append(('lrp-' + r1['gw_port_id'],
+                                          'neutron-' + r1['id']))
         self.l3_plugin.update_router(
             self.context, r1['id'],
             {'router': {'routes': [{'destination': '10.10.0.0/24',
                                     'nexthop': '20.0.0.10'},
                                    {'destination': '10.11.0.0/24',
                                     'nexthop': '20.0.0.11'}]}})
+        r1_f1 = self.l3_plugin.create_floatingip(
+            self.context, {'floatingip': {
+                'tenant_id': self._tenant_id,
+                'floating_network_id': e1['network']['id'],
+                'floating_ip_address': '100.0.0.20',
+                'port_id': n1_port_dict['p1']}})
+        r1_f2 = self.l3_plugin.create_floatingip(
+            self.context, {'floatingip': {
+                'tenant_id': self._tenant_id,
+                'floating_network_id': e1['network']['id'],
+                'floating_ip_address': '100.0.0.21'}})
+        self.l3_plugin.update_floatingip(
+            self.context, r1_f2['id'], {'floatingip': {
+                'port_id': n1_port_dict['p2']}})
+
+        # update External subnet gateway ip
+        data = {'subnet': {'gateway_ip': '100.0.0.1'}}
+        subnet_req = self.new_update_request(
+            'subnets', data, e1_s1['subnet']['id'])
+        subnet_req.get_response(self.api)
+
+        # Static routes
         self.create_lrouter_routes.append(('neutron-' + r1['id'],
                                            '10.12.0.0/24',
                                            '20.0.0.12'))
@@ -296,6 +367,37 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         self.delete_lrouter_routes.append(('neutron-' + r1['id'],
                                            '10.10.0.0/24',
                                            '20.0.0.10'))
+        # Gateway default route
+        self.delete_lrouter_routes.append(('neutron-' + r1['id'],
+                                           '0.0.0.0/0',
+                                           '100.0.0.1'))
+        # Gateway sNATs
+        self.create_lrouter_nats.append(('neutron-' + r1['id'],
+                                         '100.0.0.100',
+                                         '200.0.0.0/24', 'snat'))
+        self.delete_lrouter_nats.append(('neutron-' + r1['id'],
+                                         '100.0.0.2',
+                                         '10.0.0.0/24', 'snat'))
+        # Floating IPs
+        self.create_lrouter_nats.append(('neutron-' + r1['id'],
+                                         '100.0.0.200',
+                                         '200.0.0.200', 'dnat_and_snat'))
+        self.delete_lrouter_nats.append(('neutron-' + r1['id'],
+                                         r1_f1['floating_ip_address'],
+                                         r1_f1['fixed_ip_address'],
+                                         'dnat_and_snat'))
+
+        n4 = self._make_network(self.fmt, 'n4', True)
+        res = self._create_subnet(self.fmt, n4['network']['id'],
+                                  '40.0.0.0/24', enable_dhcp=False)
+        n4_s1 = self.deserialize(self.fmt, res)
+        n4_port_dict = {}
+        for p in ['p1', 'p2', 'p3']:
+            port = self._make_port(self.fmt, n4['network']['id'],
+                                   name='n4-' + p,
+                                   device_owner='compute:None')
+            n4_port_dict[p] = port['port']['id']
+            self.lport_dhcp_ignored.append(port['port']['id'])
 
         r2 = self.l3_plugin.create_router(
             self.context,
@@ -305,10 +407,40 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                                   name='n1-p-rtr')
         self.l3_plugin.add_router_interface(
             self.context, r2['id'], {'port_id': n1_prtr['port']['id']})
+        self.l3_plugin.add_router_interface(
+            self.context, r2['id'], {'subnet_id': n4_s1['subnet']['id']})
         self.l3_plugin.update_router(
             self.context, r2['id'],
             {'router': {'routes': [{'destination': '10.20.0.0/24',
-                                    'nexthop': '10.0.0.20'}]}})
+                                    'nexthop': '10.0.0.20'}],
+                        'external_gateway_info': {
+                        'enable_snat': False,
+                        'network_id': e1['network']['id'],
+                        'external_fixed_ips': [
+                            {'ip_address': '100.0.0.3',
+                             'subnet_id': e1_s1['subnet']['id']}]}}})
+        self.l3_plugin.create_floatingip(
+            self.context, {'floatingip': {
+                'tenant_id': self._tenant_id,
+                'floating_network_id': e1['network']['id'],
+                'floating_ip_address': '100.0.0.30',
+                'port_id': n4_port_dict['p1']}})
+        self.l3_plugin.create_floatingip(
+            self.context, {'floatingip': {
+                'tenant_id': self._tenant_id,
+                'floating_network_id': e1['network']['id'],
+                'floating_ip_address': '100.0.0.31',
+                'port_id': n4_port_dict['p2']}})
+        # To test l3_plugin.disassociate_floatingips, associating floating IP
+        # to port p3 and then deleting p3.
+        self.l3_plugin.create_floatingip(
+            self.context, {'floatingip': {
+                'tenant_id': self._tenant_id,
+                'floating_network_id': e1['network']['id'],
+                'floating_ip_address': '100.0.0.32',
+                'port_id': n4_port_dict['p3']}})
+        self._delete('ports', n4_port_dict['p3'])
+
         self.create_lrouters.append('neutron-' + uuidutils.generate_uuid())
         self.create_lrouter_ports.append(('lrp-' + uuidutils.generate_uuid(),
                                           'neutron-' + r1['id']))
@@ -332,21 +464,34 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         res = self._create_subnet(self.fmt, n3['network']['id'],
                                   '30.0.0.0/24')
         n3_s1 = self.deserialize(self.fmt, res)
+        res = self._create_subnet(self.fmt, n3['network']['id'],
+                                  '2001:dbc::/64', ip_version=6)
+        n3_s2 = self.deserialize(self.fmt, res)
+        if not restart_ovsdb_processes:
+            # Test using original mac when syncing.
+            dhcp_mac_v4 = self.mech_driver._nb_ovn.get_subnet_dhcp_options(
+                n3_s1['subnet']['id']).get('options', {}).get('server_mac')
+            dhcp_mac_v6 = self.mech_driver._nb_ovn.get_subnet_dhcp_options(
+                n3_s2['subnet']['id']).get('options', {}).get('server_id')
+            self.assertTrue(dhcp_mac_v4 is not None)
+            self.assertTrue(dhcp_mac_v6 is not None)
+            self.match_old_mac_dhcp_subnets.append(n3_s1['subnet']['id'])
+            self.match_old_mac_dhcp_subnets.append(n3_s2['subnet']['id'])
+        else:
+            dhcp_mac_v4 = '01:02:03:04:05:06'
+            dhcp_mac_v6 = '01:02:03:04:05:06'
         self.expected_dhcp_options_rows.append({
             'cidr': '30.0.0.0/24',
             'external_ids': {'subnet_id': n3_s1['subnet']['id']},
             'options': {'server_id': '30.0.0.1',
-                        'server_mac': '01:02:03:04:05:06',
+                        'server_mac': dhcp_mac_v4,
                         'lease_time': str(12 * 60 * 60),
                         'mtu': str(n3['network']['mtu']),
                         'router': n3_s1['subnet']['gateway_ip']}})
-        res = self._create_subnet(self.fmt, n3['network']['id'],
-                                  '2001:dbc::/64', ip_version=6)
-        n3_s2 = self.deserialize(self.fmt, res)
         self.expected_dhcp_options_rows.append({
             'cidr': '2001:dbc::/64',
             'external_ids': {'subnet_id': n3_s2['subnet']['id']},
-            'options': {'server_id': '01:02:03:04:05:06'}})
+            'options': {'server_id': dhcp_mac_v6}})
         fake_port_id1 = uuidutils.generate_uuid()
         fake_port_id2 = uuidutils.generate_uuid()
         self.create_lswitch_ports.append(('neutron-' + fake_port_id1,
@@ -358,7 +503,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
             'port_id': fake_port_id1,
             'cidr': '30.0.0.0/24',
             'options': {'server_id': '30.0.0.254',
-                        'server_mac': '01:02:03:04:05:06',
+                        'server_mac': dhcp_mac_v4,
                         'lease_time': str(3 * 60 * 60),
                         'mtu': str(n3['network']['mtu'] / 2),
                         'router': '30.0.0.254',
@@ -379,7 +524,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
             'subnet_id': n3_s2['subnet']['id'],
             'port_id': fake_port_id1,
             'cidr': '2001:dbc::/64',
-            'options': {'server_id': '01:02:03:04:05:06',
+            'options': {'server_id': dhcp_mac_v6,
                         'domain-search': 'foo-domain'},
             'external_ids': {'subnet_id': n3_s2['subnet']['id'],
                              'port_id': fake_port_id1},
@@ -437,7 +582,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
 
             for lrport, lrouter_name, networks in self.update_lrouter_ports:
                 txn.add(cmd.UpdateLRouterPortCommand(
-                    fake_api, lrport, lrouter_name, True,
+                    fake_api, lrport, True,
                     **{'networks': [networks]}))
 
             for lrport, lrouter_name in self.delete_lrouter_ports:
@@ -453,6 +598,18 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                 txn.add(cmd.DelStaticRouteCommand(fake_api, lrouter_name,
                                                   ip_prefix, nexthop, True))
 
+            for lrouter_name, external_ip, logical_ip, nat_type in(
+                    self.create_lrouter_nats):
+                txn.add(cmd.AddNATRuleInLRouterCommand(
+                    fake_api, lrouter_name, external_ip=external_ip,
+                    logical_ip=logical_ip, type=nat_type))
+
+            for lrouter_name, external_ip, logical_ip, nat_type in(
+                    self.delete_lrouter_nats):
+                txn.add(cmd.DeleteNATRuleInLRouterCommand(
+                    fake_api, lrouter_name, external_ip=external_ip,
+                    logical_ip=logical_ip, type=nat_type, if_exists=True))
+
             for acl in self.create_acls:
                 txn.add(cmd.AddACLCommand(fake_api, **acl))
 
@@ -534,6 +691,9 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
     def _validate_networks(self, should_match=True):
         db_networks = self._list('networks')
         db_net_ids = [net['id'] for net in db_networks['networks']]
+        db_provnet_ports = [utils.ovn_provnet_port_name(net['id'])
+                            for net in db_networks['networks']
+                            if net.get('provider:physical_network')]
 
         # Get the list of lswitch ids stored in the OVN plugin IDL
         _plugin_nb_ovn = self.mech_driver._nb_ovn
@@ -546,9 +706,21 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
             row.name.replace('neutron-', '') for row in (
                 self.monitor_nb_db_idl.tables['Logical_Switch'].rows.values())]
 
+        # Get the list of provnet ports stored in the OVN plugin IDL
+        plugin_provnet_ports = [row.name for row in (
+            _plugin_nb_ovn._tables['Logical_Switch_Port'].rows.values())
+            if row.name.startswith(ovn_const.OVN_PROVNET_PORT_NAME_PREFIX)]
+
+        # Get the list of provnet ports stored in the monitor IDL connection
+        monitor_provnet_ports = [row.name for row in (
+            self.monitor_nb_db_idl.tables['Logical_Switch_Port'].rows.values())
+            if row.name.startswith(ovn_const.OVN_PROVNET_PORT_NAME_PREFIX)]
+
         if should_match:
             self.assertItemsEqual(db_net_ids, plugin_lswitch_ids)
             self.assertItemsEqual(db_net_ids, monitor_lswitch_ids)
+            self.assertItemsEqual(db_provnet_ports, plugin_provnet_ports)
+            self.assertItemsEqual(db_provnet_ports, monitor_provnet_ports)
         else:
             self.assertRaises(
                 AssertionError, self.assertItemsEqual, db_net_ids,
@@ -558,18 +730,30 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                 AssertionError, self.assertItemsEqual, db_net_ids,
                 monitor_lswitch_ids)
 
+            self.assertRaises(
+                AssertionError, self.assertItemsEqual, db_provnet_ports,
+                plugin_provnet_ports)
+
+            self.assertRaises(
+                AssertionError, self.assertItemsEqual, db_provnet_ports,
+                monitor_provnet_ports)
+
     def _validate_ports(self, should_match=True):
         db_ports = self._list('ports')
-        db_port_ids = [port['id'] for port in db_ports['ports']]
+        db_port_ids = [port['id'] for port in db_ports['ports'] if
+                       port['device_owner'] !=
+                       constants.DEVICE_OWNER_FLOATINGIP]
         db_port_ids_dhcp_valid = set(
             port['id'] for port in db_ports['ports']
             if not port['device_owner'].startswith(
-                constants.DEVICE_OWNER_PREFIXES))
+                constants.DEVICE_OWNER_PREFIXES) and
+            port['id'] not in self.lport_dhcp_ignored)
 
         _plugin_nb_ovn = self.mech_driver._nb_ovn
         plugin_lport_ids = [
             row.name for row in (
-                _plugin_nb_ovn._tables['Logical_Switch_Port'].rows.values())]
+                _plugin_nb_ovn._tables['Logical_Switch_Port'].rows.values())
+            if ovn_const.OVN_PORT_NAME_EXT_ID_KEY in row.external_ids]
         plugin_lport_ids_dhcpv4_enabled = [
             row.name for row in (
                 _plugin_nb_ovn._tables['Logical_Switch_Port'].rows.values())
@@ -582,7 +766,8 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         monitor_lport_ids = [
             row.name for row in (
                 self.monitor_nb_db_idl.tables['Logical_Switch_Port'].
-                rows.values())]
+                rows.values())
+            if ovn_const.OVN_PORT_NAME_EXT_ID_KEY in row.external_ids]
         monitor_lport_ids_dhcpv4_enabled = [
             row.name for row in (
                 _plugin_nb_ovn._tables['Logical_Switch_Port'].rows.values())
@@ -633,10 +818,12 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         _plugin_nb_ovn = self.mech_driver._nb_ovn
         for row in _plugin_nb_ovn._tables['DHCP_Options'].rows.values():
             opts = dict(row.options)
-            if 'server_mac' in opts:
-                opts['server_mac'] = '01:02:03:04:05:06'
-            else:
-                opts['server_id'] = '01:02:03:04:05:06'
+            ids = dict(row.external_ids)
+            if ids.get('subnet_id') not in self.match_old_mac_dhcp_subnets:
+                if 'server_mac' in opts:
+                    opts['server_mac'] = '01:02:03:04:05:06'
+                else:
+                    opts['server_id'] = '01:02:03:04:05:06'
             observed_plugin_dhcp_options_rows.append({
                 'cidr': row.cidr, 'external_ids': row.external_ids,
                 'options': opts})
@@ -644,10 +831,12 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         observed_monitor_dhcp_options_rows = []
         for row in self.monitor_nb_db_idl.tables['DHCP_Options'].rows.values():
             opts = dict(row.options)
-            if 'server_mac' in opts:
-                opts['server_mac'] = '01:02:03:04:05:06'
-            else:
-                opts['server_id'] = '01:02:03:04:05:06'
+            ids = dict(row.external_ids)
+            if ids.get('subnet_id') not in self.match_old_mac_dhcp_subnets:
+                if 'server_mac' in opts:
+                    opts['server_mac'] = '01:02:03:04:05:06'
+                else:
+                    opts['server_id'] = '01:02:03:04:05:06'
             observed_monitor_dhcp_options_rows.append({
                 'cidr': row.cidr, 'external_ids': row.external_ids,
                 'options': opts})
@@ -722,11 +911,31 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         db_routers = self._list('routers')
         db_router_ids = []
         db_routes = {}
+        db_nats = {}
         for db_router in db_routers['routers']:
             db_router_ids.append(db_router['id'])
             db_routes[db_router['id']] = [db_route['destination'] +
                                           db_route['nexthop']
                                           for db_route in db_router['routes']]
+            db_nats[db_router['id']] = []
+            if db_router.get(l3.EXTERNAL_GW_INFO):
+                r_ip, gw_ip = self.l3_plugin.\
+                    get_external_router_and_gateway_ip(self.context, db_router)
+                # Add gateway default route and snats
+                if gw_ip:
+                    db_routes[db_router['id']].append('0.0.0.0/0' + gw_ip)
+                if r_ip and utils.is_snat_enabled(db_router):
+                    networks = self.l3_plugin.\
+                        _get_v4_network_of_all_router_ports(self.context,
+                                                            db_router['id'])
+                    db_nats[db_router['id']].extend([r_ip + network + 'snat'
+                                                     for network in networks])
+        fips = self._list('floatingips')
+        for fip in fips['floatingips']:
+            if fip['router_id']:
+                db_nats[fip['router_id']].append(fip['floating_ip_address'] +
+                                                 fip['fixed_ip_address'] +
+                                                 'dnat_and_snat')
 
         _plugin_nb_ovn = self.mech_driver._nb_ovn
         plugin_lrouter_ids = [
@@ -754,9 +963,12 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                                  query_params='device_id=%s' % (router_id))
             r_port_ids = [p['id'] for p in r_ports['ports']]
             r_port_networks = {
-                p['id']: self.l3_plugin.get_networks_for_lrouter_port(
-                    self.context, p['fixed_ips']) for p in r_ports['ports']}
+                p['id']:
+                    self.l3_plugin._ovn_client._get_networks_for_router_port(
+                        p['fixed_ips'])
+                    for p in r_ports['ports']}
             r_routes = db_routes[router_id]
+            r_nats = db_nats[router_id]
 
             try:
                 lrouter = idlutils.row_by_value(
@@ -771,9 +983,13 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                 sroutes = getattr(lrouter, 'static_routes', [])
                 plugin_routes = [sroute.ip_prefix + sroute.nexthop
                                  for sroute in sroutes]
+                nats = getattr(lrouter, 'nat', [])
+                plugin_nats = [nat.external_ip + nat.logical_ip + nat.type
+                               for nat in nats]
             except idlutils.RowNotFound:
                 plugin_lrouter_port_ids = []
                 plugin_routes = []
+                plugin_nats = []
 
             try:
                 lrouter = idlutils.row_by_value(
@@ -788,9 +1004,13 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                 sroutes = getattr(lrouter, 'static_routes', [])
                 monitor_routes = [sroute.ip_prefix + sroute.nexthop
                                   for sroute in sroutes]
+                nats = getattr(lrouter, 'nat', [])
+                monitor_nats = [nat.external_ip + nat.logical_ip + nat.type
+                                for nat in nats]
             except idlutils.RowNotFound:
                 monitor_lrouter_port_ids = []
                 monitor_routes = []
+                monitor_nats = []
 
             if should_match:
                 self.assertItemsEqual(r_port_ids, plugin_lrouter_port_ids)
@@ -803,6 +1023,8 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                                           monitor_lport_networks[p])
                 self.assertItemsEqual(r_routes, plugin_routes)
                 self.assertItemsEqual(r_routes, monitor_routes)
+                self.assertItemsEqual(r_nats, plugin_nats)
+                self.assertItemsEqual(r_nats, monitor_nats)
             else:
                 self.assertRaises(
                     AssertionError, self.assertItemsEqual, r_port_ids,
@@ -831,17 +1053,30 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                     AssertionError, self.assertItemsEqual, r_routes,
                     monitor_routes)
 
+                self.assertRaises(
+                    AssertionError, self.assertItemsEqual, r_nats,
+                    plugin_nats)
+
+                self.assertRaises(
+                    AssertionError, self.assertItemsEqual, r_nats,
+                    monitor_nats)
+
     def _validate_address_sets(self, should_match=True):
         db_ports = self._list('ports')['ports']
+        sgs = self._list('security-groups')['security_groups']
         db_sgs = {}
+        for sg in sgs:
+            for ip_version in ['ip4', 'ip6']:
+                name = utils.ovn_addrset_name(sg['id'], ip_version)
+                db_sgs[name] = []
+
         for port in db_ports:
-            sg_ids = port.get('security_groups', [])
+            sg_ids = utils.get_lsp_security_groups(port)
             addresses = acl_utils.acl_port_ips(port)
             for sg_id in sg_ids:
                 for ip_version in addresses:
                     name = utils.ovn_addrset_name(sg_id, ip_version)
-                    addr_list = db_sgs.setdefault(name, [])
-                    addr_list.extend(addresses[ip_version])
+                    db_sgs[name].extend(addresses[ip_version])
 
         _plugin_nb_ovn = self.mech_driver._nb_ovn
         nb_address_sets = _plugin_nb_ovn.get_address_sets()
@@ -872,17 +1107,12 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
     def _sync_resources(self, mode):
         nb_synchronizer = ovn_db_sync.OvnNbSynchronizer(
             self.plugin, self.mech_driver._nb_ovn, mode, self.mech_driver)
-
-        ctx = context.get_admin_context()
-        nb_synchronizer.sync_address_sets(ctx)
-        nb_synchronizer.sync_networks_ports_and_dhcp_opts(ctx)
-        nb_synchronizer.sync_acls(ctx)
-        nb_synchronizer.sync_routers_and_rports(ctx)
+        nb_synchronizer.do_sync()
 
     def _test_ovn_nb_sync_helper(self, mode, modify_resources=True,
                                  restart_ovsdb_processes=False,
                                  should_match_after_sync=True):
-        self._create_resources()
+        self._create_resources(restart_ovsdb_processes)
         self._validate_resources(should_match=True)
 
         if modify_resources:
@@ -1000,3 +1230,23 @@ class TestOvnSbSync(base.TestOVNFunctionalBase):
         # host1 should be cleared since it is not in the chassis DB. host3
         # should be cleared since there is no segment for mapping.
         self.assertEqual({'host2', 'host4'}, segment_hosts)
+
+
+class TestOvnNbSyncOverTcp(TestOvnNbSync):
+    def get_ovsdb_server_protocol(self):
+        return 'tcp'
+
+
+class TestOvnSbSyncOverTcp(TestOvnSbSync):
+    def get_ovsdb_server_protocol(self):
+        return 'tcp'
+
+
+class TestOvnNbSyncOverSsl(TestOvnNbSync):
+    def get_ovsdb_server_protocol(self):
+        return 'ssl'
+
+
+class TestOvnSbSyncOverSsl(TestOvnSbSync):
+    def get_ovsdb_server_protocol(self):
+        return 'ssl'
