@@ -13,8 +13,8 @@
 #
 
 import copy
-import mock
 
+import mock
 from neutron_lib import constants as const
 
 from networking_ovn.common import acl as ovn_acl
@@ -45,7 +45,7 @@ class TestACLs(base.TestCase):
             'cidr': '1.1.1.0/24',
         }).info()
         patcher = mock.patch(
-            'neutron.agent.ovsdb.native.idlutils.row_by_value',
+            'ovsdbapp.backend.ovs_idl.idlutils.row_by_value',
             lambda *args, **kwargs: mock.MagicMock())
         patcher.start()
 
@@ -72,7 +72,10 @@ class TestACLs(base.TestCase):
                 self.assertEqual(acl_from_lport, acl)
 
     def test_add_acl_dhcp(self):
-        acls = ovn_acl.add_acl_dhcp(self.fake_port, self.fake_subnet)
+        ovn_dhcp_acls = ovn_acl.add_acl_dhcp(self.fake_port, self.fake_subnet)
+        other_dhcp_acls = ovn_acl.add_acl_dhcp(self.fake_port,
+                                               self.fake_subnet,
+                                               ovn_dhcp=False)
 
         expected_match_to_lport = (
             'outport == "%s" && ip4 && ip4.src == %s && udp && udp.src == 67 '
@@ -85,7 +88,7 @@ class TestACLs(base.TestCase):
                         'match': expected_match_to_lport, 'priority': 1002}
         expected_match_from_lport = (
             'inport == "%s" && ip4 && '
-            '(ip4.dst == 255.255.255.255 || ip4.dst == %s) && '
+            'ip4.dst == {255.255.255.255, %s} && '
             'udp && udp.src == 68 && udp.dst == 67'
         ) % (self.fake_port['id'], self.fake_subnet['cidr'])
         acl_from_lport = {'action': 'allow', 'direction': 'from-lport',
@@ -93,7 +96,10 @@ class TestACLs(base.TestCase):
                           'log': False, 'lport': 'fake_port_id1',
                           'lswitch': 'neutron-network_id1',
                           'match': expected_match_from_lport, 'priority': 1002}
-        for acl in acls:
+        self.assertEqual(1, len(ovn_dhcp_acls))
+        self.assertEqual(acl_from_lport, ovn_dhcp_acls[0])
+        self.assertEqual(2, len(other_dhcp_acls))
+        for acl in other_dhcp_acls:
             if 'to-lport' in acl.values():
                 self.assertEqual(acl_to_lport, acl)
             if 'from-lport' in acl.values():
@@ -312,7 +318,7 @@ class TestACLs(base.TestCase):
             match='outport == port-id1 && ip4 && (ip4.src == fake_ip)')
         lswitch_obj = mock.Mock(
             name='neutron-lswitch-1', acls=[acl1, acl2, acl3])
-        with mock.patch('neutron.agent.ovsdb.native.idlutils.row_by_value',
+        with mock.patch('ovsdbapp.backend.ovs_idl.idlutils.row_by_value',
                         return_value=lswitch_obj):
             update_cmd_del_acl = cmd.UpdateACLsCommand(self.driver._nb_ovn,
                                                        [lswitch_name],
@@ -327,7 +333,7 @@ class TestACLs(base.TestCase):
             self.assertEqual(expected_acls, acl_del_dict)
             self.assertEqual({}, acl_add_dict)
 
-    def test_acl_protocol_and_ports_for_tcp_and_udp_number(self):
+    def test_acl_protocol_and_ports_for_tcp_udp_and_sctp_number(self):
         sg_rule = {'port_range_min': None,
                    'port_range_max': None}
 
@@ -339,6 +345,10 @@ class TestACLs(base.TestCase):
         match = ovn_acl.acl_protocol_and_ports(sg_rule, None)
         self.assertEqual(' && udp', match)
 
+        sg_rule['protocol'] = str(const.PROTO_NUM_SCTP)
+        match = ovn_acl.acl_protocol_and_ports(sg_rule, None)
+        self.assertEqual(' && sctp', match)
+
     def test_acl_protocol_and_ports_for_ipv6_icmp_protocol(self):
         sg_rule = {'port_range_min': None,
                    'port_range_max': None}
@@ -393,6 +403,41 @@ class TestACLs(base.TestCase):
             match = ovn_acl.acl_protocol_and_ports(sg_rule, icmp)
             self.assertEqual(expected_match, match)
 
+    def test_acl_protocol_and_ports_protocol_not_supported(self):
+        sg_rule = {'port_range_min': None,
+                   'port_range_max': None}
+        sg_rule['protocol'] = '1234567'
+        self.assertRaises(ovn_acl.ProtocolNotSupported,
+                          ovn_acl.acl_protocol_and_ports, sg_rule, None)
+
+    def test_acl_protocol_and_ports_protocol_range(self):
+        sg_rule = {'port_range_min': None,
+                   'port_range_max': None}
+
+        # For more common protocols such as TCP, UDP and ICMP, we
+        # prefer to use the protocol name in the match string instead of
+        # the protocol number (e.g: the word "tcp" instead of "ip.proto
+        # == 6"). This improves the readability/debbugability when
+        # troubleshooting the ACLs
+        skip_protos = (const.PROTO_NUM_TCP, const.PROTO_NUM_UDP,
+                       const.PROTO_NUM_SCTP, const.PROTO_NUM_ICMP,
+                       const.PROTO_NUM_IPV6_ICMP)
+
+        for proto in range(256):
+            if proto in skip_protos:
+                continue
+            sg_rule['protocol'] = str(proto)
+            match = ovn_acl.acl_protocol_and_ports(sg_rule, None)
+            self.assertEqual(' && ip.proto == %s' % proto, match)
+
+    def test_acl_protocol_and_ports_name_to_number(self):
+        sg_rule = {'port_range_min': None,
+                   'port_range_max': None}
+
+        sg_rule['protocol'] = str(const.PROTO_NAME_OSPF)
+        match = ovn_acl.acl_protocol_and_ports(sg_rule, None)
+        self.assertEqual(' && ip.proto == 89', match)
+
     def test_acl_direction(self):
         sg_rule = fakes.FakeSecurityGroupRule.create_one_security_group_rule({
             'direction': 'ingress'
@@ -424,8 +469,8 @@ class TestACLs(base.TestCase):
         sg_rule['ethertype'] = 'IPv10'
         match, ip_version, icmp = ovn_acl.acl_ethertype(sg_rule)
         self.assertEqual('', match)
-        self.assertEqual(None, ip_version)
-        self.assertEqual(None, icmp)
+        self.assertIsNone(ip_version)
+        self.assertIsNone(icmp)
 
     def test_acl_remote_ip_prefix(self):
         sg_rule = fakes.FakeSecurityGroupRule.create_one_security_group_rule({
