@@ -14,7 +14,7 @@
 
 import mock
 
-from neutron.agent.ovsdb.native import idlutils
+from ovsdbapp.backend.ovs_idl import idlutils
 
 from networking_ovn.common import acl as ovn_acl
 from networking_ovn.common import constants as ovn_const
@@ -329,34 +329,55 @@ class TestSetLSwitchPortCommand(TestBaseCommand):
             cmd.run_idl(self.transaction)
             self.assertEqual(new_ext_ids, fake_lsp.external_ids)
 
-    def _test_lswitch_port_update_del_dhcp(self, has_v4_opts, has_v6_opts):
+    def _test_lswitch_port_update_del_dhcp(self, clear_v4_opts,
+                                           clear_v6_opts, set_v4_opts=False,
+                                           set_v6_opts=False):
         ext_ids = {ovn_const.OVN_PORT_NAME_EXT_ID_KEY: 'test'}
-        new_ext_ids = {ovn_const.OVN_PORT_NAME_EXT_ID_KEY: 'test-new'}
         dhcp_options_tbl = self.ovn_api._tables['DHCP_Options']
-        if has_v4_opts:
-            fake_dhcpv4_opts = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-                attrs={'external_ids': {'port_id': 'fake-lsp'}})
-            dhcp_options_tbl.rows[fake_dhcpv4_opts.uuid] = fake_dhcpv4_opts
-        if has_v6_opts:
-            fake_dhcpv6_opts = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-                attrs={'external_ids': {'port_id': 'fake-lsp'}})
-            dhcp_options_tbl.rows[fake_dhcpv6_opts.uuid] = fake_dhcpv6_opts
+        fake_dhcpv4_opts = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'external_ids': {'port_id': 'fake-lsp'}})
+        dhcp_options_tbl.rows[fake_dhcpv4_opts.uuid] = fake_dhcpv4_opts
+        fake_dhcpv6_opts = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'external_ids': {'port_id': 'fake-lsp'}})
+        dhcp_options_tbl.rows[fake_dhcpv6_opts.uuid] = fake_dhcpv6_opts
         fake_lsp = fakes.FakeOvsdbRow.create_one_ovsdb_row(
             attrs={'name': 'fake-lsp',
                    'external_ids': ext_ids,
-                   'dhcpv4_options': has_v4_opts and [fake_dhcpv4_opts] or [],
-                   'dhcpv6_options': has_v6_opts and [fake_dhcpv6_opts] or []})
+                   'dhcpv4_options': [fake_dhcpv4_opts],
+                   'dhcpv6_options': [fake_dhcpv6_opts]})
+
+        columns = {}
+        if clear_v4_opts:
+            columns['dhcpv4_options'] = []
+        elif set_v4_opts:
+            columns['dhcpv4_options'] = [fake_dhcpv4_opts.uuid]
+        if clear_v6_opts:
+            columns['dhcpv6_options'] = []
+        elif set_v6_opts:
+            columns['dhcpv6_options'] = [fake_dhcpv6_opts.uuid]
+
         with mock.patch.object(idlutils, 'row_by_value',
                                return_value=fake_lsp):
             cmd = commands.SetLSwitchPortCommand(
-                self.ovn_api, fake_lsp.name, if_exists=True,
-                external_ids=new_ext_ids)
+                self.ovn_api, fake_lsp.name, if_exists=True, **columns)
             cmd.run_idl(self.transaction)
-            self.assertEqual(new_ext_ids, fake_lsp.external_ids)
-            if has_v4_opts:
+
+            if clear_v4_opts and clear_v6_opts:
                 fake_dhcpv4_opts.delete.assert_called_once_with()
-            if has_v6_opts:
                 fake_dhcpv6_opts.delete.assert_called_once_with()
+            elif clear_v4_opts:
+                # not clear_v6_opts and set_v6_opts is any
+                fake_dhcpv4_opts.delete.assert_called_once_with()
+                fake_dhcpv6_opts.delete.assert_not_called()
+            elif clear_v6_opts:
+                # not clear_v4_opts and set_v6_opts is any
+                fake_dhcpv4_opts.delete.assert_not_called()
+                fake_dhcpv6_opts.delete.assert_called_once_with()
+            else:
+                # not clear_v4_opts and not clear_v6_opts and
+                # set_v4_opts is any and set_v6_opts is any
+                fake_dhcpv4_opts.delete.assert_not_called()
+                fake_dhcpv6_opts.delete.assert_not_called()
 
     def test_lswitch_port_update_del_port_dhcpv4_options(self):
         self._test_lswitch_port_update_del_dhcp(True, False)
@@ -367,6 +388,19 @@ class TestSetLSwitchPortCommand(TestBaseCommand):
     def test_lswitch_port_update_del_all_port_dhcp_options(self):
         self._test_lswitch_port_update_del_dhcp(True, True)
 
+    def test_lswitch_port_update_del_no_port_dhcp_options(self):
+        self._test_lswitch_port_update_del_dhcp(False, False)
+
+    def test_lswitch_port_update_set_port_dhcpv4_options(self):
+        self._test_lswitch_port_update_del_dhcp(False, True, set_v4_opts=True)
+
+    def test_lswitch_port_update_set_port_dhcpv6_options(self):
+        self._test_lswitch_port_update_del_dhcp(True, False, set_v6_opts=True)
+
+    def test_lswitch_port_update_set_all_port_dhcp_options(self):
+        self._test_lswitch_port_update_del_dhcp(False, False, set_v4_opts=True,
+                                                set_v6_opts=True)
+
     def _test_lswitch_port_update_with_dhcp(self, dhcpv4_opts, dhcpv6_opts):
         ext_ids = {ovn_const.OVN_PORT_NAME_EXT_ID_KEY: 'test'}
         fake_lsp = fakes.FakeOvsdbRow.create_one_ovsdb_row(
@@ -648,7 +682,7 @@ class TestUpdateLRouterPortCommand(TestBaseCommand):
         with mock.patch.object(idlutils, 'row_by_value',
                                side_effect=idlutils.RowNotFound):
             cmd = commands.UpdateLRouterPortCommand(
-                self.ovn_api, 'fake-lrp', 'fake-lrouter', if_exists=if_exists)
+                self.ovn_api, 'fake-lrp', if_exists=if_exists)
             if if_exists:
                 cmd.run_idl(self.transaction)
             else:
@@ -668,7 +702,7 @@ class TestUpdateLRouterPortCommand(TestBaseCommand):
         with mock.patch.object(idlutils, 'row_by_value',
                                return_value=fake_lrp):
             cmd = commands.UpdateLRouterPortCommand(
-                self.ovn_api, fake_lrp.name, 'fake-lrouter', if_exists=True,
+                self.ovn_api, fake_lrp.name, if_exists=True,
                 networks=new_networks)
             cmd.run_idl(self.transaction)
             self.assertEqual(new_networks, fake_lrp.networks)
@@ -719,19 +753,27 @@ class TestSetLRouterPortInLSwitchPortCommand(TestBaseCommand):
         with mock.patch.object(idlutils, 'row_by_value',
                                side_effect=idlutils.RowNotFound):
             cmd = commands.SetLRouterPortInLSwitchPortCommand(
-                self.ovn_api, 'fake-lsp', 'fake-lrp')
+                self.ovn_api, 'fake-lsp', 'fake-lrp', if_exists=False)
             self.assertRaises(RuntimeError, cmd.run_idl, self.transaction)
 
+    def test_lswitch_port_no_exist_do_not_fail(self):
+        with mock.patch.object(idlutils, 'row_by_value',
+                               side_effect=idlutils.RowNotFound):
+            cmd = commands.SetLRouterPortInLSwitchPortCommand(
+                self.ovn_api, 'fake-lsp', 'fake-lrp', if_exists=True)
+            cmd.run_idl(self.transaction)
+
     def test_lswitch_port_router_update(self):
         lrp_name = 'fake-lrp'
         fake_lsp = fakes.FakeOvsdbRow.create_one_ovsdb_row()
         with mock.patch.object(idlutils, 'row_by_value',
                                return_value=fake_lsp):
             cmd = commands.SetLRouterPortInLSwitchPortCommand(
-                self.ovn_api, fake_lsp.name, lrp_name)
+                self.ovn_api, fake_lsp.name, lrp_name, if_exists=True)
             cmd.run_idl(self.transaction)
             self.assertEqual({'router-port': lrp_name}, fake_lsp.options)
             self.assertEqual('router', fake_lsp.type)
+            self.assertEqual('router', fake_lsp.addresses)
 
 
 class TestAddACLCommand(TestBaseCommand):
