@@ -13,8 +13,10 @@
 #
 
 import copy
+
 import mock
 
+from networking_ovn.common import config
 from networking_ovn.common import constants as ovn_const
 from networking_ovn.common import utils
 from networking_ovn.ovsdb import impl_idl_ovn
@@ -90,6 +92,9 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                               'lsp-rp-name-1'},
              'options': {'router-port':
                          utils.ovn_lrouter_port_name('orp-id-a1')}},
+            {'name': 'provnet-ls-id-1', 'addresses': ['unknown'],
+             'external_ids': {},
+             'options': {'network_name': 'physnet1'}},
             {'name': 'lsp-id-21', 'addresses': ['10.0.2.1'],
              'external_ids': {ovn_const.OVN_PORT_NAME_EXT_ID_KEY:
                               'lsp-name-21'}},
@@ -102,6 +107,9 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                               'lsp-rp-name-2'},
              'options': {'router-port':
                          utils.ovn_lrouter_port_name('orp-id-a2')}},
+            {'name': 'provnet-ls-id-2', 'addresses': ['unknown'],
+             'external_ids': {},
+             'options': {'network_name': 'physnet2'}},
             {'name': 'lsp-id-31', 'addresses': ['10.0.3.1'],
              'external_ids': {ovn_const.OVN_PORT_NAME_EXT_ID_KEY:
                               'lsp-name-31'}},
@@ -138,39 +146,48 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         'lrouters': [
             {'name': utils.ovn_name('lr-id-a'),
              'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
-                              'lr-name-a'},
-             'options': {'chassis': 'host-1'}},
+                              'lr-name-a'}},
             {'name': utils.ovn_name('lr-id-b'),
              'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
-                              'lr-name-b'},
-             'options': {'chassis': 'host-1'}},
+                              'lr-name-b'}},
             {'name': utils.ovn_name('lr-id-c'),
              'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
-                              'lr-name-c'},
-             'options': {'chassis': 'host-2'}},
+                              'lr-name-c'}},
             {'name': utils.ovn_name('lr-id-d'),
              'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
-                              'lr-name-d'},
-             'options': {'chassis': ovn_const.OVN_GATEWAY_INVALID_CHASSIS}},
+                              'lr-name-d'}},
             {'name': utils.ovn_name('lr-id-e'),
              'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
-                              'lr-name-e'},
-             'options': {}}],
+                              'lr-name-e'}}],
         'lrouter_ports': [
             {'name': utils.ovn_lrouter_port_name('orp-id-a1'),
-             'external_ids': {}, 'networks': ['10.0.1.0/24']},
+             'external_ids': {}, 'networks': ['10.0.1.0/24'],
+             'options': {ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-1'}},
             {'name': utils.ovn_lrouter_port_name('orp-id-a2'),
-             'external_ids': {}, 'networks': ['10.0.2.0/24']},
+             'external_ids': {}, 'networks': ['10.0.2.0/24'],
+             'options': {ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-1'}},
             {'name': utils.ovn_lrouter_port_name('orp-id-a3'),
-             'external_ids': {}, 'networks': ['10.0.3.0/24']},
+             'external_ids': {}, 'networks': ['10.0.3.0/24'],
+             'options': {ovn_const.OVN_GATEWAY_CHASSIS_KEY:
+                         ovn_const.OVN_GATEWAY_INVALID_CHASSIS}},
             {'name': 'xrp-id-b1',
              'external_ids': {}, 'networks': ['20.0.1.0/24']},
             {'name': utils.ovn_lrouter_port_name('orp-id-b2'),
-             'external_ids': {}, 'networks': ['20.0.2.0/24']}],
+             'external_ids': {}, 'networks': ['20.0.2.0/24'],
+             'options': {ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-2'}},
+            {'name': utils.ovn_lrouter_port_name('orp-id-b3'),
+             'external_ids': {}, 'networks': ['20.0.3.0/24'],
+             'options': {}}],
         'static_routes': [{'ip_prefix': '20.0.0.0/16',
                            'nexthop': '10.0.3.253'},
                           {'ip_prefix': '10.0.0.0/16',
                            'nexthop': '20.0.2.253'}],
+        'nats': [{'external_ip': '10.0.3.1', 'logical_ip': '20.0.0.0/16',
+                  'type': 'snat'},
+                 {'external_ip': '20.0.2.1', 'logical_ip': '10.0.0.0/24',
+                  'type': 'snat'},
+                 {'external_ip': '20.0.2.4', 'logical_ip': '10.0.0.4',
+                  'type': 'dnat_and_snat'}],
         'acls': [
             {'unit_test_id': 1,
              'action': 'allow-related', 'direction': 'from-lport',
@@ -219,10 +236,10 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             {'cidr': '10.0.2.0/24',
              'external_ids': {'subnet_id': 'subnet-id-10-0-2-0'},
              'options': {'mtu': '1442', 'router': '10.0.2.254'}},
-            {'cidr': '10.0.3.0/24',
-             'external_ids': {'subnet_id': 'subnet-id-10-0-3-0',
+            {'cidr': '10.0.1.0/26',
+             'external_ids': {'subnet_id': 'subnet-id-10-0-1-0',
                               'port_id': 'lsp-vpn-id-3'},
-             'options': {'mtu': '1442', 'router': '10.0.3.254'}},
+             'options': {'mtu': '1442', 'router': '10.0.1.1'}},
             {'cidr': '20.0.1.0/24',
              'external_ids': {'subnet_id': 'subnet-id-20-0-1-0'},
              'options': {'mtu': '1442', 'router': '20.0.1.254'}},
@@ -259,9 +276,10 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
     fake_associations = {
         'lstolsp': {
             utils.ovn_name('ls-id-1'): [
-                'lsp-id-11', 'lsp-id-12', 'lsp-rp-id-1'],
+                'lsp-id-11', 'lsp-id-12', 'lsp-rp-id-1', 'provnet-ls-id-1'],
             utils.ovn_name('ls-id-2'): [
-                'lsp-id-21', 'lsp-id-22', 'lsp-id-23', 'lsp-rp-id-2'],
+                'lsp-id-21', 'lsp-id-22', 'lsp-id-23', 'lsp-rp-id-2',
+                'provnet-ls-id-2'],
             utils.ovn_name('ls-id-3'): [
                 'lsp-id-31', 'lsp-id-32', 'lsp-rp-id-3', 'lsp-vpn-id-3'],
             'ls-id-4': [
@@ -280,6 +298,10 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             utils.ovn_name('lr-id-a'): ['20.0.0.0/16'],
             utils.ovn_name('lr-id-b'): ['10.0.0.0/16']
             },
+        'lrtonat': {
+            utils.ovn_name('lr-id-a'): ['10.0.3.1'],
+            utils.ovn_name('lr-id-b'): ['20.0.2.1', '20.0.2.4'],
+            },
         'lstoacl': {
             utils.ovn_name('ls-id-1'): [1, 2, 3, 4],
             utils.ovn_name('ls-id-2'): [5, 6],
@@ -295,6 +317,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         self.lrouter_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
         self.lrp_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
         self.sroute_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
+        self.nat_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
         self.acl_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
         self.dhcp_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
         self.address_set_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
@@ -312,7 +335,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         with mock.patch.object(impl_idl_ovn, 'get_connection',
                                return_value=mock.Mock()):
             impl_idl_ovn.OvsdbNbOvnIdl.ovsdb_connection = None
-            self.nb_ovn_idl = impl_idl_ovn.OvsdbNbOvnIdl(self)
+            self.nb_ovn_idl = impl_idl_ovn.OvsdbNbOvnIdl(mock.Mock())
 
         self.nb_ovn_idl.idl.tables = self._tables
 
@@ -345,6 +368,14 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             TestNBImplIdlOvn.fake_associations['lrtosroute'],
             self.lrouter_table, self.sroute_table,
             'name', 'ip_prefix', 'static_routes')
+        # Load nats
+        fake_nats = TestNBImplIdlOvn.fake_set['nats']
+        self._load_ovsdb_fake_rows(self.nat_table, fake_nats)
+        # Associate routers and nats
+        self._construct_ovsdb_references(
+            TestNBImplIdlOvn.fake_associations['lrtonat'],
+            self.lrouter_table, self.nat_table,
+            'name', 'external_ip', 'nat')
         # Load acls
         fake_acls = TestNBImplIdlOvn.fake_set['acls']
         self._load_ovsdb_fake_rows(self.acl_table, fake_acls)
@@ -360,6 +391,20 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         fake_address_sets = TestNBImplIdlOvn.fake_set['address_sets']
         self._load_ovsdb_fake_rows(self.address_set_table, fake_address_sets)
 
+    @mock.patch.object(impl_idl_ovn.OvsdbNbOvnIdl, 'ovsdb_connection', None)
+    @mock.patch.object(impl_idl_ovn, 'get_connection', mock.Mock())
+    def test_setting_ovsdb_probe_timeout_default_value(self):
+        inst = impl_idl_ovn.OvsdbNbOvnIdl(mock.Mock())
+        inst.idl._session.reconnect.set_probe_interval.assert_called_with(0)
+
+    @mock.patch.object(impl_idl_ovn.OvsdbNbOvnIdl, 'ovsdb_connection', None)
+    @mock.patch.object(impl_idl_ovn, 'get_connection', mock.Mock())
+    @mock.patch.object(config, 'get_ovn_ovsdb_probe_interval')
+    def test_setting_ovsdb_probe_timeout(self, mock_get_probe_interval):
+        mock_get_probe_interval.return_value = 5000
+        inst = impl_idl_ovn.OvsdbNbOvnIdl(mock.Mock())
+        inst.idl._session.reconnect.set_probe_interval.assert_called_with(5000)
+
     def test_get_all_logical_switches_with_ports(self):
         # Test empty
         mapping = self.nb_ovn_idl.get_all_logical_switches_with_ports()
@@ -368,15 +413,19 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         self._load_nb_db()
         mapping = self.nb_ovn_idl.get_all_logical_switches_with_ports()
         expected = [{'name': utils.ovn_name('ls-id-1'),
-                     'ports': ['lsp-id-11', 'lsp-id-12', 'lsp-rp-id-1']},
+                     'ports': ['lsp-id-11', 'lsp-id-12', 'lsp-rp-id-1'],
+                     'provnet_port': 'provnet-ls-id-1'},
                     {'name': utils.ovn_name('ls-id-2'),
-                     'ports': ['lsp-id-21', 'lsp-rp-id-2']},
+                     'ports': ['lsp-id-21', 'lsp-rp-id-2'],
+                     'provnet_port': 'provnet-ls-id-2'},
                     {'name': utils.ovn_name('ls-id-3'),
                      'ports': ['lsp-id-31', 'lsp-id-32', 'lsp-rp-id-3',
-                               'lsp-vpn-id-3']},
+                               'lsp-vpn-id-3'],
+                     'provnet_port': None},
                     {'name': utils.ovn_name('ls-id-5'),
                      'ports': ['lsp-id-51', 'lsp-id-52', 'lsp-rp-id-5',
-                               'lsp-vpn-id-5']}]
+                               'lsp-vpn-id-5'],
+                     'provnet_port': None}]
         self.assertItemsEqual(mapping, expected)
 
     def test_get_all_logical_routers_with_rports(self):
@@ -391,15 +440,28 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                                'orp-id-a2': ['10.0.2.0/24'],
                                'orp-id-a3': ['10.0.3.0/24']},
                      'static_routes': [{'destination': '20.0.0.0/16',
-                                        'nexthop': '10.0.3.253'}]},
+                                        'nexthop': '10.0.3.253'}],
+                     'snats': [{'external_ip': '10.0.3.1',
+                                'logical_ip': '20.0.0.0/16',
+                                'type': 'snat'}],
+                     'dnat_and_snats': []},
                     {'name': 'lr-id-b',
                      'ports': {'xrp-id-b1': ['20.0.1.0/24'],
                                'orp-id-b2': ['20.0.2.0/24']},
                      'static_routes': [{'destination': '10.0.0.0/16',
-                                        'nexthop': '20.0.2.253'}]},
-                    {'name': 'lr-id-c', 'ports': {}, 'static_routes': []},
-                    {'name': 'lr-id-d', 'ports': {}, 'static_routes': []},
-                    {'name': 'lr-id-e', 'ports': {}, 'static_routes': []}]
+                                        'nexthop': '20.0.2.253'}],
+                     'snats': [{'external_ip': '20.0.2.1',
+                                'logical_ip': '10.0.0.0/24',
+                                'type': 'snat'}],
+                     'dnat_and_snats': [{'external_ip': '20.0.2.4',
+                                         'logical_ip': '10.0.0.4',
+                                         'type': 'dnat_and_snat'}]},
+                    {'name': 'lr-id-c', 'ports': {}, 'static_routes': [],
+                     'snats': [], 'dnat_and_snats': []},
+                    {'name': 'lr-id-d', 'ports': {}, 'static_routes': [],
+                     'snats': [], 'dnat_and_snats': []},
+                    {'name': 'lr-id-e', 'ports': {}, 'static_routes': [],
+                     'snats': [], 'dnat_and_snats': []}]
         self.assertItemsEqual(mapping, expected)
 
     def test_get_acls_for_lswitches(self):
@@ -469,63 +531,68 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         self.assertEqual(len(acl_objs), 0)
         self.assertEqual(len(lswitch_ovsdb_dict), 0)
 
-    def test_get_all_chassis_router_bindings(self):
+    def test_get_all_chassis_gateway_bindings(self):
         self._load_nb_db()
-        bindings = self.nb_ovn_idl.get_all_chassis_router_bindings()
-        expected = {'host-1': [utils.ovn_name('lr-id-a'),
-                               utils.ovn_name('lr-id-b')],
-                    'host-2': [utils.ovn_name('lr-id-c')],
+        bindings = self.nb_ovn_idl.get_all_chassis_gateway_bindings()
+        expected = {'host-1': [utils.ovn_lrouter_port_name('orp-id-a1'),
+                               utils.ovn_lrouter_port_name('orp-id-a2')],
+                    'host-2': [utils.ovn_lrouter_port_name('orp-id-b2')],
                     ovn_const.OVN_GATEWAY_INVALID_CHASSIS: [
-                        utils.ovn_name('lr-id-d')]}
+                        utils.ovn_name('orp-id-a3')]}
         self.assertItemsEqual(bindings, expected)
 
-        bindings = self.nb_ovn_idl.get_all_chassis_router_bindings([])
+        bindings = self.nb_ovn_idl.get_all_chassis_gateway_bindings([])
         self.assertItemsEqual(bindings, expected)
 
-        bindings = self.nb_ovn_idl.get_all_chassis_router_bindings(['host-1'])
-        expected = {'host-1': [utils.ovn_name('lr-id-a'),
-                               utils.ovn_name('lr-id-b')]}
+        bindings = self.nb_ovn_idl.get_all_chassis_gateway_bindings(['host-1'])
+        expected = {'host-1': [utils.ovn_lrouter_port_name('orp-id-a1'),
+                               utils.ovn_lrouter_port_name('orp-id-a2')]}
         self.assertItemsEqual(bindings, expected)
 
-    def test_get_router_chassis_binding(self):
+    def test_get_gateway_chassis_binding(self):
         self._load_nb_db()
-        chassis = self.nb_ovn_idl.get_router_chassis_binding(
-            utils.ovn_name('lr-id-a'))
+        chassis = self.nb_ovn_idl.get_gateway_chassis_binding(
+            utils.ovn_lrouter_port_name('orp-id-a1'))
         self.assertEqual(chassis, 'host-1')
-        chassis = self.nb_ovn_idl.get_router_chassis_binding(
-            utils.ovn_name('lr-id-c'))
+        chassis = self.nb_ovn_idl.get_gateway_chassis_binding(
+            utils.ovn_lrouter_port_name('orp-id-b2'))
         self.assertEqual(chassis, 'host-2')
-        chassis = self.nb_ovn_idl.get_router_chassis_binding(
-            utils.ovn_name('lr-id-d'))
-        self.assertEqual(chassis, None)
-        chassis = self.nb_ovn_idl.get_router_chassis_binding(
-            utils.ovn_name('lr-id-e'))
-        self.assertEqual(chassis, None)
-        chassis = self.nb_ovn_idl.get_router_chassis_binding('bad')
-        self.assertEqual(chassis, None)
+        chassis = self.nb_ovn_idl.get_gateway_chassis_binding(
+            utils.ovn_lrouter_port_name('orp-id-a3'))
+        self.assertIsNone(chassis)
+        chassis = self.nb_ovn_idl.get_gateway_chassis_binding(
+            utils.ovn_lrouter_port_name('orp-id-b3'))
+        self.assertIsNone(chassis)
+        chassis = self.nb_ovn_idl.get_gateway_chassis_binding('bad')
+        self.assertIsNone(chassis)
 
-    def test_get_unhosted_routers(self):
+    def test_get_unhosted_gateways(self):
         self._load_nb_db()
         # Test only host-1 in the valid list
-        unhosted_routers = self.nb_ovn_idl.get_unhosted_routers(['host-1'])
-        expected = {utils.ovn_name('lr-id-c'): 'host-2',
-                    utils.ovn_name('lr-id-d'):
-                    {'chassis': ovn_const.OVN_GATEWAY_INVALID_CHASSIS}}
-        self.assertItemsEqual(unhosted_routers, expected)
+        unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(['host-1'])
+        expected = {
+            utils.ovn_lrouter_port_name('orp-id-b2'): {
+                ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-2'},
+            utils.ovn_lrouter_port_name('orp-id-a3'): {
+                ovn_const.OVN_GATEWAY_CHASSIS_KEY:
+                    ovn_const.OVN_GATEWAY_INVALID_CHASSIS}}
+        self.assertItemsEqual(unhosted_gateways, expected)
         # Test both host-1, host-2 in valid list
-        unhosted_routers = self.nb_ovn_idl.get_unhosted_routers(['host-1',
-                                                                 'host-2'])
-        expected = {utils.ovn_name('lr-id-d'):
-                    {'chassis': ovn_const.OVN_GATEWAY_INVALID_CHASSIS}}
-        self.assertItemsEqual(unhosted_routers, expected)
-        # Schedule unhosted_routers on host-2
-        for unhosted_router in unhosted_routers:
-            router_row = self._find_ovsdb_fake_row(self.lrouter_table,
-                                                   'name', unhosted_router)
-            setattr(router_row, 'options', {'chassis': 'host-2'})
-        unhosted_routers = self.nb_ovn_idl.get_unhosted_routers(['host-1',
-                                                                 'host-2'])
-        self.assertItemsEqual(unhosted_routers, {})
+        unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(['host-1',
+                                                                   'host-2'])
+        expected = {utils.ovn_lrouter_port_name('orp-id-a3'): {
+            ovn_const.OVN_GATEWAY_CHASSIS_KEY:
+                ovn_const.OVN_GATEWAY_INVALID_CHASSIS}}
+        self.assertItemsEqual(unhosted_gateways, expected)
+        # Schedule unhosted_gateways on host-2
+        for unhosted_gateway in unhosted_gateways:
+            router_row = self._find_ovsdb_fake_row(self.lrp_table,
+                                                   'name', unhosted_gateway)
+            setattr(router_row, 'options', {
+                ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-2'})
+        unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(['host-1',
+                                                                  'host-2'])
+        self.assertItemsEqual(unhosted_gateways, {})
 
     def test_get_subnet_dhcp_options(self):
         self._load_nb_db()
@@ -545,6 +612,39 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             'port-id-30-0-1-0')
         self.assertIsNone(subnet_options)
 
+    def test_get_subnet_and_ports_dhcp_options(self):
+        # Test empty
+        subnet_options = self.nb_ovn_idl.get_subnet_and_ports_dhcp_options(
+            'subnet-id-10-0-1-0')
+        self.assertItemsEqual([], subnet_options)
+        # Test loaded values
+        self._load_nb_db()
+        # Test getting both subnet and port dhcp options
+        subnet_options = self.nb_ovn_idl.get_subnet_and_ports_dhcp_options(
+            'subnet-id-10-0-1-0')
+        dhcp_rows = [
+            self._find_ovsdb_fake_row(self.dhcp_table, 'cidr', '10.0.1.0/24'),
+            self._find_ovsdb_fake_row(self.dhcp_table, 'cidr', '10.0.1.0/26')]
+        expected_rows = [{'cidr': dhcp_row.cidr,
+                          'external_ids': dhcp_row.external_ids,
+                          'options': dhcp_row.options,
+                          'uuid': dhcp_row.uuid} for dhcp_row in dhcp_rows]
+        self.assertItemsEqual(expected_rows, subnet_options)
+        # Test getting only subnet dhcp options
+        subnet_options = self.nb_ovn_idl.get_subnet_and_ports_dhcp_options(
+            'subnet-id-10-0-2-0')
+        dhcp_rows = [
+            self._find_ovsdb_fake_row(self.dhcp_table, 'cidr', '10.0.2.0/24')]
+        expected_rows = [{'cidr': dhcp_row.cidr,
+                          'external_ids': dhcp_row.external_ids,
+                          'options': dhcp_row.options,
+                          'uuid': dhcp_row.uuid} for dhcp_row in dhcp_rows]
+        self.assertItemsEqual(expected_rows, subnet_options)
+        # Test getting no dhcp options
+        subnet_options = self.nb_ovn_idl.get_subnet_and_ports_dhcp_options(
+            'subnet-id-11-0-2-0')
+        self.assertItemsEqual([], subnet_options)
+
     def test_get_subnets_dhcp_options(self):
         self._load_nb_db()
         get_row_dict = lambda row: {
@@ -578,7 +678,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         self.assertEqual(len(dhcp_options['ports_v4']), 2)
 
     def test_compose_dhcp_options_commands(self):
-        # TODO(azbiswas): Implement in seperate patch
+        # TODO(azbiswas): Implement in separate patch
         pass
 
     def test_get_address_sets(self):
@@ -596,7 +696,7 @@ class TestSBImplIdlOvn(TestDBImplIdlOvn):
                               'public:br-ex,private:br-0'}},
             {'name': 'host-2', 'hostname': 'host-2.localdomain.com',
              'external_ids': {'ovn-bridge-mappings':
-                              'public:br-ex'}},
+                              'public:br-ex,public2:br-ex'}},
             {'name': 'host-3', 'hostname': 'host-3.localdomain.com',
              'external_ids': {'ovn-bridge-mappings':
                               'public:br-ex'}},
@@ -613,7 +713,7 @@ class TestSBImplIdlOvn(TestDBImplIdlOvn):
         with mock.patch.object(impl_idl_ovn, 'get_connection',
                                return_value=mock.Mock()):
             impl_idl_ovn.OvsdbSbOvnIdl.ovsdb_connection = None
-            self.sb_ovn_idl = impl_idl_ovn.OvsdbSbOvnIdl(self)
+            self.sb_ovn_idl = impl_idl_ovn.OvsdbSbOvnIdl(mock.Mock())
 
         self.sb_ovn_idl.idl.tables = self._tables
 
@@ -622,6 +722,20 @@ class TestSBImplIdlOvn(TestDBImplIdlOvn):
         fake_chassis = TestSBImplIdlOvn.fake_set['chassis']
         self._load_ovsdb_fake_rows(self.chassis_table, fake_chassis)
 
+    @mock.patch.object(impl_idl_ovn.OvsdbSbOvnIdl, 'ovsdb_connection', None)
+    @mock.patch.object(impl_idl_ovn, 'get_connection', mock.Mock())
+    def test_setting_ovsdb_probe_timeout_default_value(self):
+        inst = impl_idl_ovn.OvsdbSbOvnIdl(mock.Mock())
+        inst.idl._session.reconnect.set_probe_interval.assert_called_with(0)
+
+    @mock.patch.object(impl_idl_ovn.OvsdbSbOvnIdl, 'ovsdb_connection', None)
+    @mock.patch.object(impl_idl_ovn, 'get_connection', mock.Mock())
+    @mock.patch.object(config, 'get_ovn_ovsdb_probe_interval')
+    def test_setting_ovsdb_probe_timeout(self, mock_get_probe_interval):
+        mock_get_probe_interval.return_value = 5000
+        inst = impl_idl_ovn.OvsdbSbOvnIdl(mock.Mock())
+        inst.idl._session.reconnect.set_probe_interval.assert_called_with(5000)
+
     def test_get_chassis_hostname_and_physnets(self):
         self._load_sb_db()
         mapping = self.sb_ovn_idl.get_chassis_hostname_and_physnets()
